@@ -11,14 +11,14 @@
 import featuretools
 import featuretools as ft
 import featuretools.variable_types as vt
-from featuretools.selection import remove_low_information_features, remove_single_value_features
 
 from tqdm.auto import tqdm
-from tql.pipe import reduce_mem_usage, cprint
+from tql.pipe import cprint
+from tql.utils.pandas_utils import reduce_mem_usage, duplicate_columns
 from tql.ml.automl.primitives import *
+from featuretools.selection import remove_low_information_features, remove_single_value_features
 
 
-# todo： 增加时间特征
 class AutoFeat(object):
     vt = vt
 
@@ -32,7 +32,7 @@ class AutoFeat(object):
         self.es = ft.EntitySet(id='MAIN')
         self.es.entity_from_dataframe(
             entity_id=self.entity_id,
-            dataframe=df.copy(),
+            dataframe=df,  # df.copy()
             index=self.index,
             variable_types=self.variable_types,
             time_index=time_index,
@@ -46,8 +46,18 @@ class AutoFeat(object):
         for col in tqdm(self.normalize_entity_cols, desc='Normalize Entity'):
             self.es.normalize_entity(self.entity_id, col, col)
 
-    def run_dfs(self, max_depth=1, features_only=True, chunk_size=None, n_jobs=1, ignore_variables=None,
-                reduce_mem=False, trans_primitives=None, agg_primitives=None, **kwargs):
+    def run_dfs(
+            self, max_depth=1,
+            features_only=True,
+            ignore_variables=None,
+            reduce_mem=False,
+            reduce_feats=True,
+            trans_primitives=None,
+            agg_primitives=None,
+            chunk_size=None,
+            n_jobs=1,
+            **kwargs
+    ):
         """Deep Feature Synthesisf
         agg_primitives (list[str or AggregationPrimitive], optional): List of Aggregation
             Feature types to apply.
@@ -65,12 +75,15 @@ class AutoFeat(object):
 
         """
         if ignore_variables is None:
-            ignore_variables = [self.target_entity_id, self.index]
+            # ignore_variables = [self.target_entity_id, self.index]
+            # ignore_variables = ["__id"]  # 忽略单值id 会少了一些count特征
+            ignore_variables = []
 
         if trans_primitives is None:
             trans_primitives = [
                 "year", "month", "day", "hour", "minute", "week", "weekday", "is_weekend",
-                'time_since_previous',  # diff
+                'time_since_previous',
+                # diff # https://stackoverflow.com/questions/60324672/how-is-time-since-previous-computed-in-featuretools
                 Quarter(),
             ]
 
@@ -92,8 +105,17 @@ class AutoFeat(object):
             return _
         else:
             df_ = _[0].add_prefix(f'{self.entity_id}_').reset_index()
-            df_ = remove_low_information_features(df_)
-            df_ = remove_single_value_features(df_, count_nan_as_value=True)
+
+            if reduce_feats:
+                cprint("remove_low_information_features")
+                df_ = remove_low_information_features(df_)
+
+                cprint("remove_single_value_features")
+                df_ = remove_single_value_features(df_, count_nan_as_value=True)
+
+                cprint("remove_duplicate_features")
+                dups = duplicate_columns(df_)
+                df_ = df_.drop(dups, 1)
 
             if reduce_mem:
                 df_ = reduce_mem_usage(df_)
@@ -128,12 +150,15 @@ if __name__ == '__main__':
     import pandas as pd
 
     df = pd.DataFrame([[1, 2, 3], [2, 2, 3], [3, 2, 3]], columns=['uid', 'a', 'b'])
+    df['date'] = pd.date_range('2020-01-01', periods=3)
 
     type2features = {
         vt.Id: ['uid'],
-        vt.Categorical: ['a', 'b']
+        vt.Categorical: ['a', 'b'],
+        vt.Datetime: ['date']
     }
 
     af = AutoFeat(df, 'test', 'test', type2features, index='uid')  # base_entity 就是 base_entity_id
+    df_ft = af.run_dfs(3, False, trans_primitives=['cum_max'], reduce_feats=False)
 
-    print(af.run_dfs(max_depth=2))  # print(af.run_dfs(max_depth=2, ignore_variables=['uid']))
+    print(df_ft)
