@@ -14,6 +14,35 @@ import pandas as pd
 from tqdm.auto import tqdm
 
 
+def duplicate_columns(frame):
+    """keep='first'
+    https://stackoverflow.com/questions/14984119/python-pandas-remove-duplicate-columns/32961145#32961145
+    数据大:
+        dups = duplicate_columns(df)
+        df.drop(dups, 1)
+
+    数据小:
+        df.T.drop_duplicates().T
+    """
+    frame = frame.fillna(-123456)  # 处理缺失值
+
+    groups = frame.columns.to_series().groupby(frame.dtypes).groups
+    dups = []
+    for t, v in tqdm(groups.items(), desc="duplicate_columns"):
+        dcols = frame[v].to_dict(orient="list")
+
+        vs = list(dcols.values())
+        ks = list(dcols.keys())
+        lvs = len(vs)
+
+        for i in range(lvs):
+            for j in range(i + 1, lvs):
+                if vs[i] == vs[j]:
+                    dups.append(ks[j])  # keep='first'
+                    break
+    return dups
+
+
 # reduce memory
 def reduce_mem_usage(df):
     numerics = ['int16', 'int32', 'int64', 'float16', 'float32', 'float64']
